<?php
/**
 * Main frontend WPML compatibility controller.
 *
 * @package Toolset Forms
 * @since 2.5.5
 */

namespace OTGS\Toolset\CRED\Controller\Compatibility\Wpml;

/**
 * Compatibility layer for frontend forms and WPML.
 *
 * @since 2.5.5
 */
class Frontend {

	/**
	 * Whether WPML is active and configured.
	 *
	 * @var bool
	 */
	private $is_wpml_active_and_configured;

	/**
	 * Construcor and dependencies.
	 *
	 * @param \Toolset_Condition_Plugin_Wpml_Is_Active_And_Configured $core_condition The conditional object to check whether WPML is ready.
	 */
	public function __construct(
		\Toolset_Condition_Plugin_Wpml_Is_Active_And_Configured $core_condition = null
	) {
		$is_wpml_active_and_configured_condition = ( null !== $core_condition )
			? $core_condition
			: new \Toolset_Condition_Plugin_Wpml_Is_Active_And_Configured();
		$this->is_wpml_active_and_configured = $is_wpml_active_and_configured_condition->is_met();
	}

	/**
	 * Initalize this controller, depending on the WPML components available.
	 *
	 * @since 2.5.5
	 */
	public function initialize() {
		if ( false === $this->is_wpml_active_and_configured ) {
			return;
		}

		$this->initialize_wpml_core();
	}

	/**
	 * Initialize compatibility with the WPML core plugin.
	 *
	 * @since 2.5.5
	 */
	private function initialize_wpml_core() {
		add_action( 'cred_save_data', array( $this, 'maybe_sync_field_values_on_frontend_submit' ), 10, 2 );
	}

	/**
	 * Maybe sync all sync-able felds for a post after it is submitted in a frontend form.
	 *
	 * @param int $post_id
	 * @param array $form_data
	 * @since 2.5.5
	 */
	public function maybe_sync_field_values_on_frontend_submit( $post_id, $form_data ) {
		$form_id = (int) toolset_getarr( $form_data, 'id', 0 );
		if ( 0 === $form_id ) {
			return;
		}

		if ( \OTGS\Toolset\CRED\Controller\Forms\Post\Main::POST_TYPE !== get_post_type( $form_id ) ) {
			// Only post forms have expiration settings.
			return;
		}

		do_action( 'wpml_sync_all_custom_fields', $post_id );
	}

}
