// External dependencies
import classnames from 'classnames';
import { map } from 'lodash';

// WordPress dependencies
import { __, sprintf } from '@wordpress/i18n';
import { Button, ColorPicker, Dashicon, Dropdown, Tooltip } from '@wordpress/components';

// Internal dependencies
import { colorAsString } from '../../../utils';

export default function ColorPalette( { colors, disableCustomColors = false, value, onChange, className } ) {
	function applyOrUnset( color ) {
		return () => onChange( value === color ? undefined : color );
	}
	const customColorPickerLabel = __( 'Custom color picker', 'wpv-views' );
	return (
		<div className={ classnames(
			'components-color-palette', // WP 5.3
			'components-circular-option-picker', // "WP 5.4
			className
		) }>
			{ map( colors, ( { color, name }, index ) => {

				const style = { color: colorAsString( color ) };
				return (
					<div
						key={ 'color-' + index }
						className={ classnames(
							'components-color-palette__item-wrapper', // WP 5.3
							'components-circular-option-picker__option-wrapper', // WP 5.4
						) }
					>
						<Tooltip
							text={ name ||
                            // translators: %s: color hex code e.g: "#f00".
                            sprintf( __( 'Color code: %s', 'wpv-views' ), color )
							}>
							<button
								type="button"
								className={ classnames(
									'components-color-palette__item', // WP 5.3
									'components-button', // WP 5.4
									'components-circular-option-picker__option', // WP 5.4
									{ 'is-active': value === color },
								) }
								style={ style }
								onClick={ applyOrUnset( color ) }
								aria-label={ name ?
									sprintf( __( 'Color: %s', 'wpv-views' ), name ) :
									sprintf( __( 'Color code: %s', 'wpv-views' ), color ) }
								aria-pressed={ value === color }
							/>
						</Tooltip>
						{ value === color && <Dashicon icon="saved" /> }
					</div>
				);
			} ) }

			<div className={ classnames(
				'components-color-palette__custom-clear-wrapper', // WP 5.3
				'components-circular-option-picker__custom-clear-wrapper' // WP 5.4
			) } >
				{ ! disableCustomColors &&
				<Dropdown
					className={ classnames(
						'components-color-palette__custom-color', // WP 5.3
						'components-circular-option-picker__dropdown-link-action', // WP 5.4
					) }
					contentClassName="components-color-palette__picker"
					renderToggle={ ( { isOpen, onToggle } ) => (
						<Button
							aria-expanded={ isOpen }
							onClick={ onToggle }
							aria-label={ customColorPickerLabel }
							isLink
						>
							{ __( 'Custom Color', 'wpv-views' ) }
						</Button>
					) }
					renderContent={ () => (
						<ColorPicker
							color={ value }
							onChangeComplete={ ( color ) => onChange( color.rgb ) }
						/>
					) }
				/>
				}

				<Button
					className={ classnames(
						'components-color-palette__clear', // WP 5.3
						'components-circular-option-picker__clear' // WP 5.4
					) }
					type="button"
					onClick={ () => onChange( undefined ) }
					isSmall
					isSecondary
				>
					{ __( 'Clear', 'wpv-views' ) }
				</Button>
			</div>
		</div>
	);
}
