import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { PanelRow, RadioControl } from '@wordpress/components';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, labels, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'applyMaxWidth',
			mediaDevices,
			applyMaxWidth => applyMaxWidth === false || applyMaxWidth === true
		);

		// For Desktop (no max width) use applyMaxWidth as default value.
		// Otherwise applyMaxWidth shouldn't have a default value.
		let applyMaxWidth = ! mediaDevices[ mediaActiveDevice ].maxWidth ? 'max-width' : null;

		if ( data.applyMaxWidth === true ) {
			applyMaxWidth = 'max-width';
		} else if ( data.applyMaxWidth === false ) {
			applyMaxWidth = 'no-max-width';
		}

		const label = labels.control || false;

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					{ label && <label>{ label }</label> }
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>
				<RadioControl
					selected={ applyMaxWidth }
					options={ [
						{ label: __( 'Limit display width and height to image parents size.', 'wpv-views' ), value: 'max-width' },
						{ label: __( 'Use display width and height even if the image overflow its parents size.', 'wpv-views' ), value: 'no-max-width' },
					] }
					onChange={ ( value ) => setAttributes( { applyMaxWidth: value === 'max-width' } ) }
				/>
			</div>

		);
	}
}
