import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { ColorControl } from '../../../index';
import { getKeyOrDefault } from '../../../../../utils';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';

export default class extends Component {
	render() {
		const { data, setAttributes, labels, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const currentMediaData = getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ backgroundColor: {} }
		);

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries( data, 'backgroundColor', mediaDevices );

		return (
			<ColorControl
				label={ getKeyOrDefault( labels, 'backgroundColor', __( 'Background Color', 'wpv-views' ) ) }
				color={ currentMediaData.backgroundColor }
				onChange={ backgroundColor => setAttributes( { backgroundColor } ) }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}
