/* WordPress dependencies */
import { Component } from '@wordpress/element';

/* Internal dependencies */
import { BorderRadius, BORDER_RADIUS_DEFAULT_VALUES, BORDER_RADIUS_IN_USE } from '../../../index';
import getData from '../utils/get-data';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';

class ControlBorderRadius extends Component {
	constructor() {
		super( ...arguments );

		this.onChange = this.onChange.bind( this );
	}

	getActiveData() {
		const { data, mediaActiveDevice, pseudoClass } = this.props;

		return getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ borderRadius: BORDER_RADIUS_DEFAULT_VALUES }
		);
	}

	onChange( value, corner, linked ) {
		const { setAttributes, disabled } = this.props;
		const dataActive = this.getActiveData();

		const valueAsInt = parseInt( value, 10 );
		if ( linked ) {
			const newBorderRadius = {};

			[ 'topLeft', 'topRight', 'bottomLeft', 'bottomRight' ].map( ( keyCorner ) => {
				if ( ! disabled || ! disabled[ keyCorner ] ) {
					newBorderRadius[ keyCorner ] = valueAsInt;
				}
			} );

			setAttributes( { borderRadius: newBorderRadius } );
		} else {
			setAttributes( { borderRadius: { ...dataActive.borderRadius, [ corner ]: valueAsInt } } );
		}
	}

	render() {
		const { disabled, data, mediaDevices } = this.props;

		const currentMediaData = this.getActiveData();

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries( data, 'borderRadius', mediaDevices, BORDER_RADIUS_IN_USE );

		return (
			<BorderRadius
				borderRadius={ currentMediaData.borderRadius }
				onChange={ this.onChange } disabled={ disabled || {} }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}

ControlBorderRadius.defaultProps = {
	disabled: {},
};

export default ControlBorderRadius;
