/* WordPress dependencies */
import { Component } from '@wordpress/element';

/* Internal dependencies */
import { Border, BORDER_DEFAULT_VALUES, BORDER_IN_USE } from '../../../index';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';

export default class extends Component {
	render() {
		const { data, setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const currentMediaData = getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ border: BORDER_DEFAULT_VALUES }
		);

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries( data, 'border', mediaDevices, BORDER_IN_USE );

		return (
			<Border
				border={ currentMediaData.border }
				onChange={ border => setAttributes( { border } ) }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}
