// WordPress dependencies
import { __ } from '@wordpress/i18n';
import { Component } from '@wordpress/element';

// Internal dependencies
import { BOX_SHADOW_DEFAULT_VALUES, BOX_SHADOW_IN_USE, BoxShadow } from '../../../index';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';

export default class extends Component {
	constructor() {
		super( ...arguments );

		this.onChangeBoxShadow = this.onChangeBoxShadow.bind( this );
		this.onChangeBoxShadowColor = this.onChangeBoxShadowColor.bind( this );
		this.onChangeBoxShadowHorizontal = this.onChangeBoxShadowHorizontal.bind( this );
		this.onChangeBoxShadowVertical = this.onChangeBoxShadowVertical.bind( this );
		this.onChangeBoxShadowBlur = this.onChangeBoxShadowBlur.bind( this );
		this.onChangeBoxShadowSpread = this.onChangeBoxShadowSpread.bind( this );
	}

	getActiveData() {
		const { data, mediaActiveDevice, pseudoClass } = this.props;

		return getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ boxShadow: BOX_SHADOW_DEFAULT_VALUES }
		);
	}

	onChangeBoxShadow( enabled ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, enabled } } );
	}

	onChangeBoxShadowColor( color ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, color } } );
	}

	onChangeBoxShadowHorizontal( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, horizontal: parseInt( value, 10 ) } } );
	}

	onChangeBoxShadowVertical( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, vertical: parseInt( value, 10 ) } } );
	}

	onChangeBoxShadowBlur( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, blur: parseInt( value, 10 ) } } );
	}

	onChangeBoxShadowSpread( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { boxShadow: { ...dataActive.boxShadow, spread: parseInt( value, 10 ) } } );
	}

	render() {
		const { data, mediaDevices } = this.props;

		const currentMediaData = this.getActiveData();

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries( data, 'boxShadow', mediaDevices, BOX_SHADOW_IN_USE );

		const boxShadow = currentMediaData.boxShadow || {};

		return (
			<BoxShadow
				label={ __( 'Box Shadow', 'wpv-views' ) }
				enabled={ boxShadow.enabled }
				color={ boxShadow.color }
				horizontal={ boxShadow.horizontal }
				vertical={ boxShadow.vertical }
				blur={ boxShadow.blur }
				spread={ boxShadow.spread }
				onChange={ this.onChangeBoxShadow }
				onChangeColor={ this.onChangeBoxShadowColor }
				onChangeHorizontal={ this.onChangeBoxShadowHorizontal }
				onChangeVertical={ this.onChangeBoxShadowVertical }
				onChangeBlur={ this.onChangeBoxShadowBlur }
				onChangeSpread={ this.onChangeBoxShadowSpread }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}
