// External Dependencies
import Select from 'react-select';
import { findIndex } from 'lodash';

// WordPress Dependencies
import { Component, Fragment } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { BaseControl, Button, PanelRow } from '@wordpress/components';

// Internal Dependencies
import googleFontsJson from './google-fonts.json';
import getData from '../../utils/get-data';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';

export default class extends Component {
	fontsAsSelectOptions() {
		return googleFontsJson.items.map( ( font ) => {
			return {
				value: font.family,
				label: font.family,
			};
		} );
	}

	fontVariantAsSelectOptions( font ) {
		if ( ! font ) {
			return null;
		}

		const fontIndex = findIndex( googleFontsJson.items, { family: font } );
		if ( ! fontIndex ) {
			return null;
		}

		const variants = googleFontsJson.items[ fontIndex ].variants
			.filter( variant => ! variant.includes( 'italic' ) )
			.map( variant => {
				return { value: variant, label: variant };
			} );

		return !! variants ? variants : null;
	}

	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass,
		);

		const fontVariants = this.fontVariantAsSelectOptions( data.font );
		const fontHasVariant = data.fontVariant && findIndex( fontVariants, { value: data.fontVariant } ) >= 0;

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'font',
			mediaDevices,
			( newData ) => {
				return !! newData;
			}
		);

		return (
			<Fragment>
				<div className="components-base-control">
					<PanelRow className="tces-row-with-media-query-controls">
						{ /* eslint-disable-next-line */ }
						<label>{ __( 'Font', 'wpv-views' ) }</label>
						<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
					</PanelRow>

					<div className="tb-row-block">
						<div style={ { width: '100%', marginRight: 2 } }>
							<Select
								value={ data.font ? { value: data.font, label: data.font } : null }
								options={ this.fontsAsSelectOptions() }
								onChange={ font => setAttributes( { font: font.value } ) }
							/>
						</div>
						<Button
							style={ {
								height: '44px',
								paddingRight: '12px',
							} }
							disabled={ ! data.font }
							onClick={ () => setAttributes( { font: undefined } ) }
							isSecondary
							isSmall
						>
							{ __( 'Reset', 'wpv-views' ) }
						</Button>
					</div>
				</div>

				{ fontVariants &&
					<BaseControl label={ __( 'Font Weight', 'wpv-views' ) } >
						<Select
							value={ data.fontVariant && fontHasVariant ?
								{ value: data.fontVariant, label: data.fontVariant } :
								{ value: 'regular', label: 'regular' } }
							options={ fontVariants || [ { value: 'regular', label: 'regular' } ] }
							onChange={ variant => setAttributes( { fontVariant: variant.value } ) }
						/>
					</BaseControl>
				}
			</Fragment>
		);
	}
}
