// External dependencies
import { without, union } from 'lodash';

// Internal dependencies
import { PanelRow, Toolbar } from '@wordpress/components';
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import getData from '../../utils/get-data';
import deepmerge from 'deepmerge';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;
		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const fontWeight = data.fontWeight || null;
		const fontStyle = data.fontStyle || [];
		const textDecoration = data.textDecoration || [];

		const mediaUsagesFontWeight = getPropertyHasValueByMediaQueries(
			this.props.data,
			'fontWeight',
			mediaDevices
		);

		const mediaUsagesFontStyle = getPropertyHasValueByMediaQueries(
			this.props.data,
			'fontStyle',
			mediaDevices
		);

		const mediaUsagesTextDecoration = getPropertyHasValueByMediaQueries(
			this.props.data,
			'textDecoration',
			mediaDevices
		);

		const customMerge = () => {
			return ( a, b ) => a || b;
		};

		deepmerge( mediaUsagesFontWeight, mediaUsagesFontStyle, { customMerge } );
		const mediaUsages = deepmerge( mediaUsagesFontWeight, mediaUsagesTextDecoration, { customMerge } );

		return (
			<div className="tces-font-style">
				<div className="components-base-control">
					<PanelRow className="tces-row-with-media-query-controls">
						{ /* eslint-disable-next-line */ }
						<label>{ __( 'Font Style', 'wpv-views' ) }</label>
						<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
					</PanelRow>

					<Toolbar controls={ [ 'bold', 'italic', 'underline', 'line-through' ].map( style => {
						switch ( style ) {
							case 'bold':
								return {
									icon: 'editor-bold',
									title: __( 'Bold', 'wpv-views' ),
									isActive: fontWeight === 'bold',
									onClick: () => {
										const updated = fontWeight === 'bold' ? undefined : 'bold';

										setAttributes( { fontWeight: updated } );
									},
								};
							case 'italic':
								return {
									icon: 'editor-italic',
									title: __( 'Italic', 'wpv-views' ),
									isActive: fontStyle.includes( style ),
									onClick: () => {
										const updated = fontStyle.includes( style ) ?
											without( fontStyle, style ) :
											union( fontStyle, [ style ] );

										setAttributes( { fontStyle: updated } );
									},
								};
							case 'line-through':
							case 'underline':
								return {
									icon: `editor-${ style === 'line-through' ? 'strikethrough' : style }`,
									title: style === 'line-through' ?
										__( 'Strikethrough', 'wpv-views' ) :
										__( 'Underline', 'wpv-views' ),
									isActive: textDecoration.includes( style ),
									onClick: () => {
										const updated = textDecoration.includes( style ) ?
											without( textDecoration, style ) :
											union( textDecoration, [ style ] );

										setAttributes( { textDecoration: updated } );
									},
								};
						}
						return {
							icon: `editor-${ style === 'line-through' ? 'strikethrough' : style }`,
							title: `${ style }`,
							isActive: fontStyle.includes( style ),
							onClick: () => {
								const updated = fontStyle.includes( style ) ?
									without( fontStyle, style ) :
									union( fontStyle, [ style ] );

								setAttributes( { fontStyle: updated } );
							},
						};
					} ) } />
				</div>
			</div>
		);
	}
}
