import { toNumber } from 'lodash';

import { Button, PanelRow } from '@wordpress/components';
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { Number as NumberField } from '../../../../index';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import getData from '../../utils/get-data';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;
		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'lineHeight',
			mediaDevices
		);

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ __( 'Line Height', 'wpv-views' ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>

				<div className="tb-row-block">
					<NumberField
						label=""
						min={ 0 }
						value={ data.lineHeight || null }
						unit={ data.lineHeightUnit }
						onChange={ ( lineHeight, lineHeightUnit ) => setAttributes( {
							lineHeight: toNumber( lineHeight ),
							lineHeightUnit,
						} ) }
						units={ [ 'px', 'em', 'rem' ] }
					/>
					<Button
						style={ {
							height: '30px',
							paddingRight: '12px',
						} }
						disabled={ ! data.lineHeight }
						onClick={ () => setAttributes( { lineHeight: undefined, lineHeightUnit: 'px' } ) }
						isSecondary
						isSmall
					>
						{ __( 'Reset', 'wpv-views' ) }
					</Button>
				</div>
			</div>
		);
	}
}
