// WordPress dependencies.
import { __ } from '@wordpress/i18n';
import { FontSizePicker, PanelRow } from '@wordpress/components';
import { Component } from '@wordpress/element';

// Internal dependencies.
import { fontSizes } from '../../../../../../utils';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';

import './scss/size.scss';
import getData from '../../utils/get-data';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';

export default class extends Component {
	render() {
		const { data, setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const currentMediaData = getData(
			data,
			mediaActiveDevice,
			pseudoClass
		);

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries(
			data,
			'fontSize',
			mediaDevices,
			newData => {
				return !! newData || newData === 0;
			}
		);

		return (
			<div className="tces-font-size-picker">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ __( 'Font Size', 'wpv-views' ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>
				<FontSizePicker
					fontSizes={ fontSizes() }
					value={ currentMediaData.fontSize }
					onChange={ fontSize => setAttributes( { fontSize } ) }
				/>
			</div>
		);
	}
}
