// WordPress dependencies.
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';

// Internal dependencies.
import { ColorControl } from '../../../../index';
import { getKeyOrDefault } from '../../../../../../utils';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import getData from '../../utils/get-data';

export default class extends Component {
	render() {
		const { setAttributes, labels, controlsMapping, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass,
		);

		const attributeKey = controlsMapping.textColor ? controlsMapping.textColor : 'textColor';

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries( this.props.data, attributeKey, mediaDevices );

		return (
			<ColorControl
				label={ getKeyOrDefault( labels, 'textColor', __( 'Text Color', 'wpv-views' ) ) }
				color={ data[ attributeKey ] }
				onChange={ textColor => setAttributes( { [ attributeKey ]: textColor } ) }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}
