// WordPress dependencies
import { __ } from '@wordpress/i18n';
import { Component } from '@wordpress/element';

// Internal dependencies
import { BOX_SHADOW_DEFAULT_VALUES, BoxShadow } from '../../../../index';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import getData from '../../utils/get-data';

export default class extends Component {
	constructor() {
		super( ...arguments );

		this.onChangeTextShadow = this.onChangeTextShadow.bind( this );
		this.onChangeTextShadowColor = this.onChangeTextShadowColor.bind( this );
		this.onChangeTextShadowHorizontal = this.onChangeTextShadowHorizontal.bind( this );
		this.onChangeTextShadowVertical = this.onChangeTextShadowVertical.bind( this );
		this.onChangeTextShadowBlur = this.onChangeTextShadowBlur.bind( this );
	}

	getActiveData() {
		const { data, mediaActiveDevice, pseudoClass } = this.props;

		return getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ textShadow: BOX_SHADOW_DEFAULT_VALUES }
		);
	}

	onChangeTextShadow( enabled ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { textShadow: { ...dataActive.textShadow, enabled } } );
	}

	onChangeTextShadowColor( color ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { textShadow: { ...dataActive.textShadow, color } } );
	}

	onChangeTextShadowHorizontal( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { textShadow: { ...dataActive.textShadow, horizontal: parseInt( value, 10 ) } } );
	}

	onChangeTextShadowVertical( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { textShadow: { ...dataActive.textShadow, vertical: parseInt( value, 10 ) } } );
	}

	onChangeTextShadowBlur( value ) {
		const { setAttributes } = this.props;
		const dataActive = this.getActiveData();
		setAttributes( { textShadow: { ...dataActive.textShadow, blur: parseInt( value, 10 ) } } );
	}

	render() {
		const { mediaDevices } = this.props;

		const dataActive = this.getActiveData();
		const textShadow = dataActive.textShadow || {};

		// This is for having media device icons highlighted which have a value set.
		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'textShadow',
			mediaDevices,
			newData => newData.enabled === true
		);

		return (
			<BoxShadow
				label={ __( 'Text Shadow', 'wpv-views' ) }
				enabled={ textShadow.enabled }
				color={ textShadow.color }
				horizontal={ textShadow.horizontal }
				vertical={ textShadow.vertical }
				blur={ textShadow.blur }
				onChange={ this.onChangeTextShadow }
				onChangeColor={ this.onChangeTextShadowColor }
				onChangeHorizontal={ this.onChangeTextShadowHorizontal }
				onChangeVertical={ this.onChangeTextShadowVertical }
				onChangeBlur={ this.onChangeTextShadowBlur }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}
