// Internal dependencies
import { Button, ButtonGroup, PanelRow } from '@wordpress/components';
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import MediaQueryControlFlexRight from '../../../../media-query/control/flex-right';
import getPropertyHasValueByMediaQueries from '../../utils/get-property-has-value-by-media-queries';
import getData from '../../utils/get-data';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;
		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'textTransform',
			mediaDevices
		);

		const textTransform = data.textTransform || undefined;

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ __( 'Text Transform', 'wpv-views' ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>

				<ButtonGroup>
					<Button
						isSmall
						isPrimary={ textTransform === 'capitalize' }
						isButton
						isDefault
						onClick={ () => {
							const updated = textTransform === 'capitalize' ?
								undefined :
								'capitalize';

							setAttributes( { textTransform: updated } );
						} }>
						{ __( 'Capitalize', 'wpv-views' ) }
					</Button>
					<Button
						isSmall
						isPrimary={ textTransform === 'uppercase' }
						isButton
						isDefault
						onClick={ () => {
							const updated = textTransform === 'uppercase' ?
								undefined :
								'uppercase';

							setAttributes( { textTransform: updated } );
						} }>
						{ __( 'Uppercase', 'wpv-views' ) }
					</Button>
					<Button
						isSmall
						isPrimary={ textTransform === 'lowercase' }
						isButton
						isDefault
						onClick={ () => {
							const updated = textTransform === 'lowercase' ?
								undefined :
								'lowercase';

							setAttributes( { textTransform: updated } );
						} }>
						{ __( 'Lowercase', 'wpv-views' ) }
					</Button>
				</ButtonGroup>
			</div>
		);
	}
}
