import { Component } from '@wordpress/element';
import {
	PaddingMarginControl,
	PADDING_DEFAULT_VALUES,
	MARGIN_DEFAULT_VALUES,
	PADDING_MARGIN_IN_USE,
} from '../../../index';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';

export default class extends Component {
	render() {
		const { data, setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const currentPadding = getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ padding: PADDING_DEFAULT_VALUES }
		);

		const currentMargin = getData(
			data,
			mediaActiveDevice,
			pseudoClass,
			{ margin: MARGIN_DEFAULT_VALUES }
		);

		// This is for having media device icons highlighted which have a value set.
		const paddingInUse = getPropertyHasValueByMediaQueries( data, 'padding', mediaDevices, PADDING_MARGIN_IN_USE );
		const marginInUse = getPropertyHasValueByMediaQueries( data, 'margin', mediaDevices, PADDING_MARGIN_IN_USE );

		const mediaUsages = {};

		Object.keys( mediaDevices ).map( ( key ) => {
			return mediaUsages[ key ] = paddingInUse[ key ] || marginInUse[ key ];
		} );

		return (
			<PaddingMarginControl
				isInitiallyOpen={ false }
				margin={ currentPadding.margin }
				padding={ currentMargin.padding }
				onChange={ userInput => setAttributes( { ...userInput } ) }
				mediaQueryControl={ <MediaQueryControlFlexRight mediaUsages={ mediaUsages } /> }
			/>
		);
	}
}

