// External dependencies
import { get } from 'lodash';

// WordPress dependencies
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';

// Internal dependencies
import { Number } from '../../../index';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';
import { PanelRow } from '@wordpress/components';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'minHeight',
			mediaDevices
		);

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ __( 'Min-Height', 'wpv-views' ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>

				<Number
					label={ '' }
					value={ get( data, 'minHeight', null ) }
					unit={ get( data, 'minHeightUnit', 'px' ) }
					units={ [ 'px', 'vh', 'vw' ] }
					onChange={
						( val, unit ) => setAttributes(
							{
								minHeight: val ? parseInt( val, 10 ) : null,
								minHeightUnit: unit,
							}
						) }
				/>
			</div>
		);
	}
}
