
// External dependencies
import { get } from 'lodash';

// WordPress dependencies
import { Component } from '@wordpress/element';
import { PanelRow, Toolbar } from '@wordpress/components';
import { __ } from '@wordpress/i18n';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import { getKeyOrDefault } from '../../../../../utils';

// Internal dependencies
const BLOCK_ALIGNMENTS_CONTROLS = {
	left: {
		icon: <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M12 5V3H3v2h9zm5 4V7H3v2h14zm-5 4v-2H3v2h9zm5 4v-2H3v2h14z" /></svg>,
		title: __( 'Text Align Left', 'wpv-views' ),
	},
	center: {
		icon: <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M14 5V3H6v2h8zm3 4V7H3v2h14zm-3 4v-2H6v2h8zm3 4v-2H3v2h14z" /></svg>,
		title: __( 'Text Align Center', 'wpv-views' ),
	},
	right: {
		icon: <svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M17 5V3H8v2h9zm0 4V7H3v2h14zm0 4v-2H8v2h9zm0 4v-2H3v2h14z" /></svg>,
		title: __( 'Text Align Right', 'wpv-views' ),
	},
};

const DEFAULT_CONTROLS = [ 'left', 'center', 'right' ];

/*
 * Toolbar providing vertical alignments: top, middle, bottom.
 */
export default class extends Component {
	render() {
		const { setAttributes, controlsMapping, mediaDevices, mediaActiveDevice, labels, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const attributeKey = controlsMapping.textAlign ? controlsMapping.textAlign : 'textAlign';

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			attributeKey,
			mediaDevices
		);

		const value = get( data, attributeKey );

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ getKeyOrDefault( labels, 'textAlign', __( 'Text Alignment', 'wpv-views' ) ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>
				<Toolbar
					isCollapsed={ false }
					label={ __( 'Change text alignment', 'wpv-views' ) }
					controls={
						DEFAULT_CONTROLS.map( ( control ) => {
							return {
								...BLOCK_ALIGNMENTS_CONTROLS[ control ],
								isActive: value === control,
								onClick: () => setAttributes( { [ attributeKey ]: control } ),
							};
						} )
					}
				/>
			</div>
		);
	}
}
