// WordPress dependencies.
import { PanelRow, RangeControl } from '@wordpress/components';
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';

// Internal dependencies.
import getData from '../utils/get-data';
import getPropertyHasValueByMediaQueries from '../utils/get-property-has-value-by-media-queries';
import MediaQueryControlFlexRight from '../../../media-query/control/flex-right';

export default class extends Component {
	render() {
		const { setAttributes, mediaDevices, mediaActiveDevice, pseudoClass } = this.props;

		const data = getData(
			this.props.data,
			mediaActiveDevice,
			pseudoClass
		);

		const mediaUsages = getPropertyHasValueByMediaQueries(
			this.props.data,
			'zIndex',
			mediaDevices
		);

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ __( 'z-index', 'wpv-views' ) }</label>
					<MediaQueryControlFlexRight mediaUsages={ mediaUsages } />
				</PanelRow>
				<RangeControl
					label={ '' }
					value={ data.zIndex || '' }
					onChange={ ( val ) => setAttributes( { zIndex: ! isNaN( val ) ? parseInt( val, 10 ) : undefined } ) }
					min={ 1 }
					max={ 20 }
					help={ __( 'An image with a greater z-index value is always in front of an element with a lower value.', 'wpv-views' ) }
				/>
			</div>
		);
	}
}
