// WordPress dependencies.
import { createHigherOrderComponent } from '@wordpress/compose';
import { Component, render } from '@wordpress/element';

// Internal dependencies.
import getData from '../utils/get-data';

function withDisplayByMediaDevice( icon ) {
	return createHigherOrderComponent( ( WrappedComponent ) => {
		return class extends Component {
			componentDidUpdate() {
				this.maybeHideBlock();
			}

			componentDidMount() {
				this.maybeHideBlock();
			}

			/**
			 * Get current display state. 'true' is the default value.
			 *
			 * @return {boolean}
			 */
			getDisplay() {
				const activeData = getData( this.props.attributes.style, this.props.mediaActiveDevice, false, { display: true } );
				if ( typeof activeData.display === 'undefined' ) {
					return true;
				}

				return activeData.display !== 'none';
			}

			maybeHideBlock() {
				if ( typeof this.props.mediaActiveDevice === 'undefined' ) {
					// eslint-disable-next-line
					console.error( 'The HOC withDisplayByMediaDevice requires the property mediaActiveDevice.' );
					return;
				}

				const display = this.getDisplay();
				const { clientId } = this.props;

				if ( ! clientId ) {
					return;
				}

				const block = document.getElementById( `block-${ clientId }` );
				const blockFlagId = `block-${ clientId }-flag`;
				const blockFlag = document.getElementById( blockFlagId );

				if ( display ) {
					// Remove flag.
					if ( blockFlag ) {
						blockFlag.parentNode.removeChild( blockFlag );
					}

					// Show block.
					block.removeAttribute( 'data-tces-hidden-block' );
					return;
				}

				// Add flag.
				if ( ! blockFlag ) {
					const newBlockFlag = document.createElement( 'div' );
					newBlockFlag.setAttribute( 'id', blockFlagId );
					newBlockFlag.setAttribute( 'class', 'tces-hidden-block-flag' );

					// Add Flag to Block.
					block.insertBefore( newBlockFlag, block.childNodes[ 0 ] );
					render( icon, newBlockFlag );
				}

				// Hide element.
				block.removeAttribute( 'data-tces-hidden-block' );
				block.setAttribute( 'data-tces-hidden-block', 1 );
			}

			render() {
				const isBlockDisplayed = this.getDisplay();

				return (
					<WrappedComponent { ...this.props } isBlockDisplayed={ isBlockDisplayed } />
				);
			}
		};
	}, 'withDisplayByMediaDevice' );
}

export default withDisplayByMediaDevice;

