// WordPress dependencies
import { Component, Fragment } from '@wordpress/element';
import { withSelect } from '@wordpress/data';
import { compose } from '@wordpress/compose';

// Internal dependencies
import {
	ControlBackgroundColor,
	ControlBorder,
	ControlBorderRadius,
	ControlBoxShadow,
	ControlFontSize,
	ControlLineHeight,
	ControlLetterSpacing,
	ControlMarginPadding,
	ControlTextColor,
	ControlRotate,
	ControlZIndex,
	ControlIdCssClasses,
	ControlBackground,
	ControlFontFamily,
	ControlFontIconToolbar,
	ControlTextTransform,
	ControlTextShadow,
	ControlMinHeight,
	ControlVerticalAlign,
	ControlWidth,
	ControlHeight,
	ControlTextAlign,
	ControlDisplay,
	ControlApplyMaxWidth,
} from '../control';
import { name as storeMediaQuery } from '../../../media-query/store';

class CommonControls extends Component {
	render() {
		const {
			controls,
			data,
			dataActive,
			labels,
			setAttributes,
			passThrough,
			controlsMapping,
			mediaDevices,
			mediaActiveDevice,
			pseudoClass,
		} = this.props;

		const commonAttributes = {
			setAttributes,
			data,
			dataActive,
			mediaDevices,
			mediaActiveDevice,
			pseudoClass,
			labels,
		};

		return (
			<Fragment>
				{ controls.includes( 'idClasses' ) && <ControlIdCssClasses { ...commonAttributes } idDisabled={ passThrough.id && passThrough.id.disabled } /> }
				{ controls.includes( 'display' ) && <ControlDisplay { ...commonAttributes } /> }
				{ controls.includes( 'fontFamily' ) && <ControlFontFamily { ...commonAttributes } /> }
				{ controls.includes( 'fontSize' ) && <ControlFontSize { ...commonAttributes } /> }
				{ controls.includes( 'textAlign' ) && <ControlTextAlign { ...commonAttributes } controlsMapping={ controlsMapping } /> }
				{ controls.includes( 'lineHeight' ) && <ControlLineHeight { ...commonAttributes } /> }
				{ controls.includes( 'letterSpacing' ) && <ControlLetterSpacing { ...commonAttributes } /> }
				{ controls.includes( 'fontIconToolbar' ) && <ControlFontIconToolbar { ...commonAttributes } /> }
				{ controls.includes( 'textTransform' ) && <ControlTextTransform { ...commonAttributes } /> }

				{ controls.includes( 'textColor' ) &&
				<ControlTextColor
					{ ...commonAttributes }
					controlsMapping={ controlsMapping }
				/>
				}

				{ controls.includes( 'textShadow' ) && <ControlTextShadow { ...commonAttributes } /> }

				{ controls.includes( 'background' ) &&
				<ControlBackground
					{ ...commonAttributes }
					labels={ labels }
					passThrough={ passThrough }
				/>
				}

				{ controls.includes( 'backgroundColor' ) &&
				<ControlBackgroundColor
					{ ...commonAttributes }
					labels={ labels }
				/>
				}

				{ controls.includes( 'margin' ) && <ControlMarginPadding { ...commonAttributes } /> }

				{ controls.includes( 'border' ) && <ControlBorder { ...commonAttributes } /> }

				{ controls.includes( 'borderRadius' ) &&
				<ControlBorderRadius
					{ ...commonAttributes }
					disabled={ passThrough.borderRadius && passThrough.borderRadius.disabled ?
						passThrough.borderRadius.disabled :
						null }
				/>
				}

				{ controls.includes( 'rotate' ) && <ControlRotate { ...commonAttributes } /> }
				{ controls.includes( 'boxShadow' ) && <ControlBoxShadow { ...commonAttributes } /> }
				{ controls.includes( 'width' ) && <ControlWidth { ...commonAttributes } labels={ labels } defaultUnit={ passThrough.width && passThrough.width.defaultUnit } /> }
				{ controls.includes( 'height' ) && <ControlHeight { ...commonAttributes } labels={ labels } defaultUnit={ passThrough.height && passThrough.height.defaultUnit } /> }
				{ controls.includes( 'applyMaxWidth' ) && <ControlApplyMaxWidth { ...commonAttributes } /> }
				{ controls.includes( 'minHeight' ) && <ControlMinHeight { ...commonAttributes } /> }
				{ controls.includes( 'verticalAlign' ) && <ControlVerticalAlign { ...commonAttributes } /> }
				{ controls.includes( 'zIndex' ) && <ControlZIndex { ...commonAttributes } /> }
			</Fragment>
		);
	}
}

const CommonControlsComposed = compose( [
	withSelect( ( select ) => {
		const { getMediaDevices, getMediaActiveDevice } = select( storeMediaQuery );
		return {
			mediaDevices: getMediaDevices(),
			mediaActiveDevice: getMediaActiveDevice(),
		};
	} ),
] )( CommonControls );

export default CommonControlsComposed;
