// External dependencies.
import { get } from 'lodash';

// Internal dependencies.
import { MEDIA_QUERY_KEY_DESKTOP } from '../../../media-query/index';

/**
 * This function will return the data level of the current user selection.
 * Can be rootData, rootData[mediaDevice], rootData[mediaDevice][pseudoClass] or rootData[pseudoClass].
 *
 * Notice: The desktop device styling attributes are stored on rootData and not on rootData[desktop].
 *         Simply because desktop styling is the general styling, which was already adjustable pre responsive options.
 *
 * @param {Array} rootData
 * @param {String} mediaDevice
 * @param {String} pseudoClass
 * @param {Array|String|Object} defaultValue
 * @return {Array|String|Object}
 */
export default function( rootData, mediaDevice = false, pseudoClass = false, defaultValue = {} ) {
	if ( typeof rootData !== 'object' ) {
		return defaultValue;
	}

	let dataLevelToUse = rootData;

	if ( mediaDevice && mediaDevice !== MEDIA_QUERY_KEY_DESKTOP ) {
		dataLevelToUse = get( rootData, mediaDevice, '--not-available1--' );
	}

	if ( !! pseudoClass ) {
		dataLevelToUse = get( dataLevelToUse, pseudoClass, '--not-available1--' );
	}

	// Return data level to use.
	if ( dataLevelToUse !== '--not-available1--' ) {
		Object.keys( defaultValue ).map( ( key ) => {
			if ( dataLevelToUse[ key ] === undefined ) {
				dataLevelToUse[ key ] = defaultValue[ key ];
			}
		} );

		return dataLevelToUse;
	}

	return defaultValue;
}
