// External dependencies.
import { get } from 'lodash';

// Internal dependencies.
import { MEDIA_QUERY_KEY_DESKTOP } from '../../../media-query/index';

/**
 * This function will return the data level of the current user selection.
 * Can be rootData, rootData[device], rootData[device][pseudoClass] or rootData[pseudoClass].
 *
 * Notice: The desktop device styling attributes are stored on rootData and not on rootData[desktop].
 *         Simply because desktop styling is the general styling, which was already adjustable pre responsive options.
 *
 * @param {Array} rootData
 * @param {String} property
 * @param {Object} mediaDevices
 * @param {Function} inUseCallback
 * @return {Array}
 */
export default function( rootData, property, mediaDevices, inUseCallback = null ) {
	const propertiesOfMediaDevices = {};
	const isInUseCallbackSet = typeof inUseCallback === 'function';
	let tmpData;

	Object.keys( mediaDevices ).map( key => {
		if ( key === MEDIA_QUERY_KEY_DESKTOP ) {
			tmpData = get( rootData, property, '-----not----used-----' );
		} else {
			tmpData = get( rootData, [ key, property ], '-----not----used-----' );
		}

		if ( tmpData !== '-----not----used-----' && isInUseCallbackSet ) {
			tmpData = inUseCallback( tmpData );
		}

		propertiesOfMediaDevices[ key ] = tmpData === '-----not----used-----' ?
			false :
			!! tmpData || tmpData === 0;
	} );

	return propertiesOfMediaDevices;
}
