/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable jsx-a11y/click-events-have-key-events */
import { Component } from '@wordpress/element';
import { __ } from '@wordpress/i18n';
import { Tooltip } from '@wordpress/components';

import classnames from 'classnames';

import { TwoColumns, ThreeColumns, FourColumns, SixColumns, Columns252550, Columns255025, Columns502525, Columns3366, Columns6633 } from './preset_icons';
import { GRID_TEMPLATE_TWO_COLUMN, GRID_TEMPLATE_THREE_COLUMN, GRID_TEMPLATE_FOUR_COLUMN, GRID_TEMPLATE_SIX_COLUMN, GRID_TEMPLATE_25_25_50, GRID_TEMPLATE_25_50_25, GRID_TEMPLATE_50_25_25, GRID_TEMPLATE_33_66, GRID_TEMPLATE_66_33 } from './grid_templates';

class GridWizard extends Component {
	render() {
		const { activeTemplate } = this.props;
		return (
			<div className="tb-grid-templates">
				<div className="tb-grid-templates__container">
					<h3>{ __( 'Select Layout', 'wpv-views' ) }</h3>
					<p>{ __( 'Choose a layout to start with.', 'wpv-views' ) }</p>
					<div className="tb-grid-templates__row">
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_TWO_COLUMN === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_TWO_COLUMN ) }
						>
							<Tooltip text={ __( 'Two Columns: Equal', 'wpv-views' ) } position="top">
								<div>
									<TwoColumns />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_THREE_COLUMN === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_THREE_COLUMN ) }
						>
							<Tooltip text={ __( 'Three Columns: Equal', 'wpv-views' ) } position="top">
								<div>
									<ThreeColumns />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_33_66 === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_33_66 ) }
						>
							<Tooltip text={ __( 'Two Columns: 1:2', 'wpv-views' ) } position="top">
								<div>
									<Columns3366 />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_66_33 === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_66_33 ) }
						>
							<Tooltip text={ __( 'Two Columns: 2:1', 'wpv-views' ) } position="top">
								<div>
									<Columns6633 />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_50_25_25 === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_50_25_25 ) }
						>
							<Tooltip text={ __( 'Three Columns: 2:1:1', 'wpv-views' ) } position="top">
								<div>
									<Columns502525 />
								</div>
							</Tooltip>
						</div>
					</div>
					<div className="tb-grid-templates__row">
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_25_50_25 === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_25_50_25 ) }
						>
							<Tooltip text={ __( 'Three Columns: 1:2:1', 'wpv-views' ) } position="top">
								<div>
									<Columns255025 />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_25_25_50 === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_25_25_50 ) }
						>
							<Tooltip text={ __( 'Three Columns: 1:1:2', 'wpv-views' ) } position="top">
								<div>
									<Columns252550 />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_FOUR_COLUMN === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_FOUR_COLUMN ) }
						>
							<Tooltip text={ __( 'Four Columns: Equal', 'wpv-views' ) } position="top">
								<div>
									<FourColumns />
								</div>
							</Tooltip>
						</div>
						<div
							className={ classnames( 'tab-grid-templates__template', { active: GRID_TEMPLATE_SIX_COLUMN === activeTemplate } ) }
							onClick={ () => this.finishWizard( GRID_TEMPLATE_SIX_COLUMN ) }
						>
							<Tooltip text={ __( 'Six Columns: Equal', 'wpv-views' ) } position="top">
								<div>
									<SixColumns />
								</div>
							</Tooltip>
						</div>
					</div>
				</div>
			</div>
		);
	}

	finishWizard = ( template ) => {
		this.props.setActiveTemplate( template );
		this.buildColumnsConfig( template );
	};

	buildColumnsConfig = ( template ) => {
		let columnConfig = null;
		switch ( template ) {
			case GRID_TEMPLATE_TWO_COLUMN:
				columnConfig = [ 0.5, 0.5 ];
				break;
			case GRID_TEMPLATE_THREE_COLUMN:
				columnConfig = [ 0.3333, 0.3333, 0.3333 ];
				break;
			case GRID_TEMPLATE_FOUR_COLUMN:
				columnConfig = [ 0.25, 0.25, 0.25, 0.25 ];
				break;
			case GRID_TEMPLATE_SIX_COLUMN:
				columnConfig = [ 0.1666, 0.1666, 0.1666, 0.1666, 0.1666, 0.1666 ];
				break;
			case GRID_TEMPLATE_25_25_50:
				columnConfig = [ 0.25, 0.25, 0.5 ];
				break;
			case GRID_TEMPLATE_25_50_25:
				columnConfig = [ 0.25, 0.5, 0.25 ];
				break;
			case GRID_TEMPLATE_50_25_25:
				columnConfig = [ 0.5, 0.25, 0.25 ];
				break;
			case GRID_TEMPLATE_33_66:
				columnConfig = [ 0.335, 0.665 ];
				break;
			case GRID_TEMPLATE_66_33:
				columnConfig = [ 0.665, 0.335 ];
				break;
		}
		this.props.allocateColumns( columnConfig );
	}
}

export default GridWizard;
