import { upperFirst } from 'lodash';

class StyleRenderer {
	buildReverseColumnsStyle( id, columnsConfig ) {
		let result = [];
		const devices = [
			'phone',
			'tablet',
			'desktop',
		];
		devices.forEach( device => {
			const up = upperFirst( device );
			if ( columnsConfig[ `reverseColumns${ up }` ] && columnsConfig[ `columns${ up }` ].length > 1 ) {
				result = result.concat( [
					`.tb-grid-active-${ device } .tb-grid.tb-grid-${ id } .block-editor-block-list__layout, .tb-grid-active-${ device }.tb-view-${ id } .tb-grid {`,
					'grid-auto-flow: dense;',
					'}',
				] );
				const count = columnsConfig[ `columns${ up }` ].length;
				for ( let i = 0; i < count; i++ ) {
					result = result.concat( [
						`.tb-grid-active-${ device }  .tb-grid.tb-grid-${ id } .block-editor-block-list__layout > .tb-grid-column:nth-child(${ count }n + ${ i + 1 }), .tb-grid-active-${ device }  .tb-grid.tb-grid-${ id } .editor-block-list__layout > [data-type="toolset-blocks/grid-column"]:nth-child(${ count }n + ${ i + 1 }) , .tb-grid-active-${ device }.tb-view-${ id } .tb-grid-column:nth-child(${ count }n + ${ i + 1 }) {`,
						`grid-column: ${ count - i };`,
						'}',
					] );
				}
			}
		} );
		return result.join( '\n' );
	}

	buildRowGapStyles( parent, curProps ) {
		const result = [
			`${ parent } [data-type="toolset-blocks/grid-column"]::after, ${ parent } .view-editor-container .tb-grid-column::after {`,
			'display: block;',
			`left: ${ -curProps.columnGap / 2 }px;`,
			`right: ${ -curProps.columnGap / 2 }px;`,
			`height: ${ curProps.rowGap }px !important;`,
			'}',
			`${ parent } [data-type="toolset-blocks/grid-column"]:nth-child(${ curProps.columns.length }n - ${ curProps.columns.length - 1 }):after {`,
			`${ curProps.reverseColumns ? 'right' : 'left' }: 0 !important;`,
			'}',
			`${ parent } [data-type="toolset-blocks/grid-column"]:nth-child(${ curProps.columns.length }n):after {`,
			`${ curProps.reverseColumns ? 'left' : 'right' }: 0 !important;`,
			'}',
			`${ parent } [data-type="toolset-blocks/grid-column"]:nth-child(${ curProps.columns.length }n):nth-last-child(-n+${ curProps.columns.length + 1 }) ~ .tb-grid-column[data-type="toolset-blocks/grid-column"]::after, ${ parent } .view-editor-container .tb-grid-column:nth-child(${ curProps.columns.length }n):nth-last-child(-n+${ curProps.columns.length + 1 }) ~ .tb-grid-column::after {`,
			'display: none;',
			'}',
		].join( '\n' );
		return result;
	}

	buildStyle( id, columnsConfig ) {
		const gridTemplatePhone = columnsConfig.columnsPhone.map( w => `minmax(0, ${ w }fr)` ).join( ' ' ),
			gridTemplateTablet = columnsConfig.columnsTablet.map( w => `minmax(0, ${ w }fr)` ).join( ' ' ),
			gridTemplateDesktop = columnsConfig.columnsDesktop.map( w => `minmax(0, ${ w }fr)` ).join( ' ' );
		const result = [
			`.tb-grid-active-phone .tb-grid.tb-grid-${ id } > .block-editor-inner-blocks > .block-editor-block-list__layout, .tb-grid-active-phone.tb-view-${ id } > div > .tb-grid {`,
			`grid-column-gap: ${ columnsConfig.columnGapPhone }px !important;`,
			`grid-row-gap: ${ columnsConfig.rowGapPhone }px !important;`,
			`grid-template-columns: ${ gridTemplatePhone } !important;`,
			'}',
			`.tb-grid-active-phone .tb-grid.tb-grid-${ id } .tb-grid-column-hidden-phone, .tb-grid-active-phone.tb-view-${ id } .tb-grid .tb-grid-column-hidden-phone {`,
			'display: none',
			'}',

			`.tb-grid-active-tablet .tb-grid.tb-grid-${ id } > .block-editor-inner-blocks > .block-editor-block-list__layout, .tb-grid-active-tablet.tb-view-${ id }  > div > .tb-grid {`,
			`grid-column-gap: ${ columnsConfig.columnGapTablet }px !important;`,
			`grid-row-gap: ${ columnsConfig.rowGapTablet }px !important;`,
			`grid-template-columns: ${ gridTemplateTablet } !important;`,
			'}',
			`.tb-grid-active-tablet .tb-grid.tb-grid-${ id } .tb-grid-column-hidden-tablet, .tb-grid-active-tablet.tb-view-${ id } .tb-grid-column-hidden-tablet {`,
			'display: none',
			'}',

			`.tb-grid-active-desktop .tb-grid.tb-grid-${ id } > .block-editor-inner-blocks > .block-editor-block-list__layout, .tb-grid-active-desktop.tb-view-${ id }  > div > .tb-grid {`,
			`grid-column-gap: ${ columnsConfig.columnGapDesktop }px !important;`,
			`grid-row-gap: ${ columnsConfig.rowGapDesktop }px !important;`,
			`grid-template-columns: ${ gridTemplateDesktop } !important;`,
			'}',
			`.tb-grid-active-desktop .tb-grid.tb-grid-${ id } .tb-grid-column-hidden-desktop, .tb-grid-active-desktop.tb-view-${ id } .tb-grid-column-hidden-desktop {`,
			'display: none',
			'}',
		];
		const devices = [ 'phone', 'desktop', 'tablet' ];
		devices.forEach( device => {
			const up = upperFirst( device );
			if ( 1 === columnsConfig[ `columns${ up }` ].length && columnsConfig[ `reverseColumns${ up }` ] ) {
				const cols = columnsConfig.columnsDesktop.length;
				let delta = 0;
				for ( let i = 0; i < cols * columnsConfig.rowsDesktop; i++ ) {
					let ind = i;
					if ( ind > cols ) {
						ind = i % cols;
					}
					if ( cols === ind ) {
						ind = 0;
					}
					result.push(
						`.tb-grid-active-${ device } .tb-grid.tb-grid-${ id } .tb-grid-column:nth-child(${ i + 1 }) {`,
						`order: ${ delta + cols - ind }`,
						'}',
					);
					if ( ( i + 1 ) % cols === 0 ) {
						delta = i;
					}
				}
			}
		} );
		return result.join( '\n' ) + this.buildReverseColumnsStyle( id, columnsConfig );
	}
}

export default StyleRenderer;
