# MediaQueryControl

<img src="./_doc/media-query-devices-control.png" width="263px">

Desktop is the current active device (blue).  
Tablet is not active, but the user has set values on it (black).  
Phone is not active and has no data (grey)


## How to use:

For common styles provided through the `<StyleControl>` there is **nothing** to do.

-----
For individual controls this should be the way to go:

On Edit component:

```
import { withSelect } from '@wordpress/data';
import { compose } from '@wordpress/compose';
import { storeMediaQuery, MediaQueryControlFlexRight } from 'toolset/block/control';

class Example extends Component {
  ...
  
  /** Example of return: { desktop: true, tablet: false, phone: true } **/
  getMediaUsages() {
    return Object.keys( this.props.mediaDevices ).map( mediaDeviceKey => {
       // logic to fetch if values exists for mediaDeviceKey or not
       ...
       
       return { [ key ]: hasValue }
    }
  }
  
  render() {
    return(
      ...
      
      <PanelRow className="tces-row-with-media-query-controls">
      
        <label>
            Example Control
        </label>
        ...
        
        // Property mediaUsages tells MediaQueryControlFlexRight which devices have user data.
        <MediaQueryControlFlexRight mediaUsages={ this.getMediaUsages() }/>
        
      </PanelRow>
      				
      ...
    )
  }
}

const ExampleComposed = compose( [
    withSelect( ( select ) => {
        const { getMediaDevices, getMediaActiveDevice } = select( storeMediaQuery );
        return {
            // All registered mediaDevices.
            mediaDevices: getMediaDevices(),
            
            // Current active device (desktop / tablet / phone).           
            mediaActiveDevice: getMediaActiveDevice(),
        };
    } ),
] )( Example );
```


**mediaActiveDevice** contains the current active device. It's up to the component how the data is stored / fetched.   
**mediaDevices** contains a list of all registered media devices

```
{
   desktop: { label: Desktop, ... }
   tablet: { ... }
   phone: { ... }
}
```
**mediaDevices** should be used to build the property **mediaUsages** of **MediaQueryControlFlexRight**.


If the device are required these constants are available:
```
mport { MEDIA_QUERY_KEY_DESKTOP, MEDIA_QUERY_KEY_TABLET, MEDIA_QUERY_KEY_PHONE } from 'toolset/block/control';
