// WordPress Dependencies
import { Component } from '@wordpress/element';
import { Icon, Tooltip } from '@wordpress/components';

// Internal Dependencies
import { MEDIA_QUERY_KEY_DESKTOP } from '../index';
import './scss/control.scss';

class Abstract extends Component {
	constructor() {
		super( ...arguments );

		this.getDevicesIcons = this.getDevicesIcons.bind( this );

		this.state = {
			displayOnlyDesktop: true,
		};
	}

	onClickDevice( key ) {
		if ( key === MEDIA_QUERY_KEY_DESKTOP ) {
			this.setState( { displayOnlyDesktop: ! this.state.displayOnlyDesktop } );
		}

		if ( key !== this.props.mediaActiveDevice ) {
			// Different key than current active clicked. Update store.
			this.props.changeMediaDevice( key );
		}
	}

	getDevicesIcons( tooltipPosition = 'top' ) {
		const { mediaDevices, mediaActiveDevice, mediaControlDisplayOnlyDesktop, mediaUsages } = this.props;

		return Object.keys( mediaDevices ).map( key => {
			if ( ( mediaControlDisplayOnlyDesktop === true && this.state.displayOnlyDesktop === true ) && key !== MEDIA_QUERY_KEY_DESKTOP ) {
				return '';
			}

			// eslint-disable-next-line
			const className = key === mediaActiveDevice ? 'tces-media-device-active' :
				typeof mediaUsages === 'object' && mediaUsages[ key ] ? 'tces-media-device-has-value' : '';

			/* eslint-disable */
			return (
				<Tooltip key={ `tooltip-${ key }` } text={ mediaDevices[ key ].label } position={ tooltipPosition }>
					<a key={ key }
						className={ className }
						style={ { width: ( 25 * mediaDevices[ key ].iconContainerWidthPercentage ) + 'px' } }
						onClick={ () => this.onClickDevice( key ) }><Icon
							size={ 20 } icon={ mediaDevices[ key ].icon } /></a>
				</Tooltip>
			);
			/* eslint-enable */
		}
		);
	}
}

export default Abstract;
