const idStyleNode = 'tces-adjust-wp-columns-per-row';

function createStyleNode() {
	document.head.insertAdjacentHTML(
		'beforeend',
		'<style id=\"' + idStyleNode + '\"> </style>' );
}

function applyStyle( style ) {
	if ( ! document.getElementById( idStyleNode ) ) {
		createStyleNode();
	}

	const styleNode = document.getElementById( idStyleNode );
	styleNode.innerHTML = style;
}

function oneColumnPerRow() {
	const style = `.wp-block-columns > .editor-inner-blocks > .editor-block-list__layout {
		flex-wrap: wrap !important; 
	}
	
	.wp-block-columns > .editor-inner-blocks > .editor-block-list__layout > [data-type="core/column"] {
		flex-basis: calc(100% - 28px) !important;
		margin: 16px auto !important;
	}`;

	applyStyle( style );
}

function twoColumnsPerRow() {
	const style = `.wp-block-columns > .editor-inner-blocks > .editor-block-list__layout {
		flex-wrap: wrap !important; 
	}
	
	.wp-block-columns > .editor-inner-blocks > .editor-block-list__layout > [data-type="core/column"] {
		flex-basis: calc(50% - (16px + 28px)) !important; 
		flex-grow: 0 !important;
		margin: 16px 32px 16px 12px !important;
	}
	.wp-block-columns > .editor-inner-blocks > .editor-block-list__layout > [data-type="core/column"]:nth-child(even) {
		margin-right: 12px !important;
		margin-left: 32px !important;
	}
	`;

	applyStyle( style );
}

function defaultColumnsPerRow() {
	// Reset styling.
	applyStyle( '' );
}

export default function( state ) {
	if ( typeof state !== 'object' ||
		state === null ||
		! state.mediaActiveDevice ||
		! state.mediaDevices ||
		! state.mediaDevices[ state.mediaActiveDevice ]
	) {
		// State does not provide the required data.
		return;
	}

	const activeDeviceData = state.mediaDevices[ state.mediaActiveDevice ];

	switch ( activeDeviceData.columnsPerRow ) {
		case 2:
			twoColumnsPerRow();
			break;
		case 1:
			oneColumnPerRow();
			break;
		default:
			defaultColumnsPerRow();
	}
}
