// External dependencies
import PropTypes from 'prop-types';

// WordPress dependencies
import { Component } from '@wordpress/element';
import { PanelRow, RangeControl } from '@wordpress/components';
import { __ } from '@wordpress/i18n';

// Internal Dependencies
import { getNumberOr } from '../../../utils';

class Rotate extends Component {
	render() {
		const { label, value, onChange, min, max } = this.props;

		return (
			<div className="components-base-control">
				<PanelRow className="tces-row-with-media-query-controls">
					{ /* eslint-disable-next-line */ }
					<label>{ label || __( 'Rotate', 'wpv-views' ) }</label>
					{ this.props.mediaQueryControl }
				</PanelRow>
				<RangeControl
					label={ '' }
					value={ getNumberOr( value ) }
					onChange={ onChange }
					min={ min }
					max={ max }
				/>
			</div>
		);
	}
}

Rotate.defaultProps = {
	label: __( 'Rotate', 'wpv-views' ),
	value: undefined,
	min: -180,
	max: 180,
};

Rotate.propTypes = {
	label: PropTypes.string,
	value: PropTypes.number,
	min: PropTypes.number,
	max: PropTypes.number,
	onChange: PropTypes.func.isRequired,
};

export default Rotate;
