/**
 * Settings are spreading across nested objects, this method gets all of them.
 *
 * @param {object} fields Fields object
 * @param {object} options Result object
 * @return {object}
 */
const getInitialOptions = ( fields, options ) => {
	if ( fields.constructor !== Object ) {
		return result;
	}
	const result = Object.assign( {}, options );
	Object.keys( fields ).forEach( key => {
		if ( key === 'fields' ) {
			const newOptions = {};
			Object.keys( fields[ key ] ).forEach( optionKey => {
				newOptions[ optionKey ] = findDefault( fields[ key ], optionKey );
			} );
			Object.assign( result, newOptions );
		}
		if ( fields[ key ].constructor === Object ) {
			Object.assign( result, getInitialOptions( fields[ key ], result ) );
		}
	} );
	return result;
};

/**
 * Finds a default value from one of it's possible names in given field.
 *
 * @param {object} field
 * @param {string} optionKey
 * @return {string}
 */
const findDefault = ( field, optionKey ) => {
	for ( const defaultName of [ 'defaultValue', 'default', 'defaultForceValue', 'default_force' ] ) {
		if ( field[ optionKey ][ defaultName ] ) {
			return field[ optionKey ][ defaultName ];
		}
	}
	return '';
};

export default getInitialOptions;
