/**
 * Debounce function
 *
 * Returns a function, that, as long as it continues to be invoked, will not
 * be triggered. The function will be called after it stops being called for
 * N milliseconds. If `immediate` is passed, trigger the function on the
 * leading edge, instead of the trailing.
 *
 * @param {function} func Function
 * @param {boolean} wait Execute it now
 * @param {integer} immediate Waiting time
 * @return {function}
 * @link https://codeburst.io/throttling-and-debouncing-in-javascript-b01cad5c8edf
 */
export function debounce( func, wait, immediate ) {
	let timeout;
	return function() {
		const context = this;
		const args = arguments;
		const later = function() {
			timeout = null;
			if ( ! immediate ) {
				func.apply( context, args );
			}
		};
		const callNow = immediate && ! timeout;
		clearTimeout( timeout );
		timeout = setTimeout( later, wait );
		if ( callNow ) {
			func.apply( context, args );
		}
	};
}
