<?php

namespace ToolsetBlocks\PublicDependencies;

use ToolsetBlocks\PublicDependencies\Dependency\IGeneral;


/**
 * Backend
 *
 * @since 1.1.0
 */
class Backend {

	/** @var IGeneral[] */
	private $dependencies = array();

	/**
	 * Add a content based dependecy
	 * @param IGeneral $dependency [description]
	 */
	public function add_dependency( IGeneral $dependency ) {
		$this->dependencies[] = $dependency;
	}

	/**
	 * Load registered dependencies
	 */
	public function load_dependencies() {
		if( ! is_admin() ) {
			// no backend
			return;
		}

		foreach( $this->dependencies as $dependency ) {
			$dependency->load_dependencies();
		}

		add_filter( 'content_save_pre', [ $this, 'fix_conditional_blocks_content' ], 1000, 1 );
	}

	/**
	 * There is a problem with `<InnerBlocks>`, they add new lines between blocks. It causes that the block is marked as invalid:
	 *   ```
	 *     Content generated by `save` function:
	 *     [wpv-conditional if=" NOT ( empty( '[wpv-current-user info="id"]') AND ( '2' = '223' ) ) " debug="true"][/wpv-conditional]
	 *     Content retrieved from post body:
	 *     [wpv-conditional if=" NOT ( empty( '[wpv-current-user info="id"]') AND ( '2' = '223' ) ) " debug="true"]
	 *     [/wpv-conditional]
	 *   ```
	 * To avoid that, the post content must be modified before it is saved, removing extra lines
	 */
	public function fix_conditional_blocks_content( $content ) {
		$count = preg_match_all( '/\\[wpv-conditional((?!\\[wpv-conditional).)*\\[\\/wpv-conditional\\]/isU', $content, $matches );
		while ( $count ) {
			$content = preg_replace( '#-->[\n|\r]+<!--#isU', '--><!--', $content );
			$aux = str_replace( '[wpv-conditional', '[@@@wpv-conditional', $matches[0][0] );
			$content = str_replace( $matches[0][0], $aux, $content );
			$count = preg_match_all( '/(\\[wpv-conditional((?!\\[wpv-conditional).)*)\\[\\/wpv-conditional\\]/isU', $content, $matches );
		}
		$content = str_replace( '[@@@wpv-conditional', '[wpv-conditional', $content );
		return $content;
	}
}
