<?php

/**
 * Update old access settings when Access activated
 */
function taccess_on_activate() {
	TAccess_Loader::load( 'CLASS/Updater' );
	Access_Updater::maybeUpdate();
	$access_settings = \OTGS\Toolset\Access\Models\Settings::get_instance();
	$access_settings->remove_depricated_settings();
}


/**
 * get current post id
 *
 * @return bool|string
 */
function toolset_access_get_current_page_id() {
	// Avoid breaking CLI
	if ( ! isset( $_SERVER['HTTP_HOST'] ) || ! isset( $_SERVER['REQUEST_URI'] ) ) {
		return '';
	}

	$protocol = stripos( $_SERVER['SERVER_PROTOCOL'], 'https' ) === true ? 'https://' : 'http://';
	$url = $protocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

	$post_types = get_post_types( '', 'names' );
	$stored_post_types = Access_Cacher::get( 'wpcf-access-current-post-types' );
	if ( false === $stored_post_types ) {
		Access_Cacher::set( 'wpcf-access-current-post-types', $post_types );
		$check_post_id = true;
	} else {
		if ( $post_types == $stored_post_types ) {
			$check_post_id = false;
		} else {
			Access_Cacher::set( 'wpcf-access-current-post-types', $post_types );
			$check_post_id = true;
		}
	}

	$post_id = Access_Cacher::get( 'wpcf-access-current-post-id' );
	if ( false === $post_id || $check_post_id ) {
		global $sitepress;
		if ( is_object( $sitepress ) ) {
			remove_filter( 'url_to_postid', array( $sitepress, 'url_to_postid' ) );
			$post_id = url_to_postid( $url );
			add_filter( 'url_to_postid', array( $sitepress, 'url_to_postid' ) );
			if ( empty( $post_id ) ) {
				$post_id = url_to_postid( $url );
			}
		} else {
			$post_id = url_to_postid( $url );
		}

		if ( ! isset( $post_id ) || empty( $post_id ) || $post_id == 0 ) {
			if ( count( $_GET ) == 1 && get_option( 'permalink_structure' ) == '' ) {
				global $wpdb;
				foreach ( $_GET as $key => $val ) {
					$val = $wpdb->esc_like( $val );
					$key = $wpdb->esc_like( $key );
					if ( post_type_exists( $key ) ) {
						$post_id = $wpdb->get_var( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_name = '%s' and post_type='%s'", $val, $key ) );
					}
				}
			}
		}

		if ( empty( $post_id ) ) {
			$homepage = get_option( 'page_on_front' );
			if ( get_home_url() . '/' == $url && $homepage != '' ) {
				$post_id = $homepage;
			}
		}

		if ( ! isset( $post_id ) || empty( $post_id ) ) {
			$post_id = '';
		} else {
			Access_Cacher::set( 'wpcf-access-current-post-id', $post_id );
		}

		$post_id = Access_Cacher::get( 'wpcf-access-current-post-id' );

	}

	return $post_id;
}


/**
 * Get post type singular and plural names
 *
 * @param string $post_type
 *
 * @return array
 * @example array( 'singular_post_type_name', 'plural_post_type_name' )
 */
function toolset_access_get_post_type_names( $post_type ) {
	global $wp_post_types;
	$plural = $singular = '';
	if ( isset( $wp_post_types[ $post_type ] ) && $wp_post_types[ $post_type ]->public == 1 ) {
		// Force map meta caps, if not builtin
		if ( in_array( $post_type, array( 'post', 'page' ) ) ) {
			switch ( $post_type ) {
				case 'page':
					$singular = 'page';
					$plural = 'pages';
					break;
				case 'post':
				default:
					$singular = 'post';
					$plural = 'posts';
					break;
			}
		} else {
			// else use singular/plural names
			$singular = sanitize_title( $wp_post_types[ $post_type ]->labels->singular_name );
			$plural = sanitize_title( $wp_post_types[ $post_type ]->labels->name );
			if ( $singular == $plural ) {
				$plural = $plural . '_s';
			}
		}
	}

	return array( $plural, $singular );
}
