<?php
// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// BEGIN ENQUEUE PARENT ACTION
// AUTO GENERATED - Do not modify or remove comment markers above or below:

if ( !function_exists( 'chld_thm_cfg_locale_css' ) ):
    function chld_thm_cfg_locale_css( $uri ){
        if ( empty( $uri ) && is_rtl() && file_exists( get_template_directory() . '/rtl.css' ) )
            $uri = get_template_directory_uri() . '/rtl.css';
        return $uri;
    }
endif;
add_filter( 'locale_stylesheet_uri', 'chld_thm_cfg_locale_css' );

if ( !function_exists( 'chld_thm_cfg_parent_css' ) ):
    function chld_thm_cfg_parent_css() {
        wp_enqueue_style( 'chld_thm_cfg_parent', trailingslashit( get_template_directory_uri() ) . 'style.css', array(  ) );
    }
endif;
add_action( 'wp_enqueue_scripts', 'chld_thm_cfg_parent_css', 10 );


function my_custom_scripts() {
    wp_enqueue_script( 'custom-js', get_stylesheet_directory_uri() . '/js/tc-custom.js', array( 'jquery' ),'',true );
}
add_action( 'wp_enqueue_scripts', 'my_custom_scripts' );
// END ENQUEUE PARENT ACTION

/***************************************
* Custom logic for Hardship Calculator
* RXA / Catherine Hayes
* April 2020
***************************************/

// add available query vars so can grab from URL
function add_query_vars_filter( $vars ){
  $vars[] = "pm";
  $vars[] = "cid";
  return $vars;
}
add_filter( 'query_vars', 'add_query_vars_filter' );


// replace ORGANIZATION custom user field with list of CPT Organizations
add_filter( 'wpt_field_options', 'list_organizations', 10, 3);
 function list_organizations( $options, $title, $type ){
  switch( $title ){
    case 'Organization':
      $options = array();
      $args = array(
          'post_type'   => 'organization',
          'post_status' => 'publish',
          'orderby'     => 'title',
          'order'       => 'ASC');
      $posts_array = get_posts( $args );
      /*$options[] = array(
              '#value' => 0,
              '#title' => "Select an option...",
          );*/
      foreach ($posts_array as $post) {
          $options[] = array(
              '#value' => $post->ID,
              '#title' => $post->post_title,
          );
      }
      $options[] = array(
              '#value' => "Other",
              '#title' => "Other",
          );
      break;
  }
  return $options;
}


// custom redirect for Debtor Submission STEP 1 (post form = 17)
add_filter('cred_success_redirect', 'custom_redirect_calculator',10,3);
function custom_redirect_calculator($url, $post_id, $form_data) {

  if ($form_data['id']==17) {
    return $url . '?cid=' . $post_id;
  }
  return $url;
}


// custom validation for Debtor Submission STEP 1 (post form=17)
add_filter('cred_form_validate','custom_validate_calculator',10,2);
function custom_validate_calculator($error_fields, $form_data) {

  //field data are field values and errors
  list($fields,$errors)=$error_fields;
  //uncomment this if you want to print the field values
  //print_r($fields);
  
  if ($form_data['id']==17) {
    
    $fname = $_POST['wpcf-first-name'];
    $lname = $_POST['wpcf-last-name'];
    $address = $_POST['wpcf-address'];
    $zip_code = $_POST['wpcf-zip-code'];
    
    $test_zip_code = '98513';
    $test_lname = 'Clark';
    $test_address = '7529 BRIANNA COURT SE';	

    // this will return an array like this: ['address_key'=>'_7529_BRIANNA__98513','estimated_value'=>'335304','equity_pct'=>'29','liquid_assets'=>'75000 to 99999'] OR like this ['error'=>'some error']
    $api_data = get_api_data($test_address, $test_zip_code, $test_lname);
    
    // check if $address_key already exists in any post_meta from last 30 days
    // TODO: check if this extra hit to API costs anything?
    $address_key = $api_data['address_key'];
    $args = array(
      'post_type'   => 'debtor-submission',
      'posts_per_page' => 10,
      'meta_key'    => 'wpcf-address-key',
      'meta_value'  => $address_key,
      'date_query' => array(
        array(
          'after' => '-30 days',
          'column' => 'post_date',
        )
      ),
    );
    $postslist = get_posts( $args );

    if ($postslist) {
      // TODO: re-enable this for live site
      //$errors['wpcf-address']='You have already checked this address once in the last month. Please come back later, or try a different address.';
    }
    
    // example field validation
    /*if ( ( $fields['wpcf-section-featured']['value'] == 'yes' ) && ( empty( $fields['wpcf-section-field-2']['value'] )) ){
      $errors['section-field-2']='Please enter a response in Field 2';
    }*/
    
  }
  return array($fields,$errors);
  
} 

// custom saves for Debtor Submission STEP 1 & STEP 2 (post forms=17 & 133)
add_action('cred_save_data', 'custom_save_calculator',20,2);
function custom_save_calculator($post_id, $form_data) {

  if ($form_data['id']==17) {
    
    // perhaps break apart into org + username? eg. /calculator?pm=associa/username
    // for now, use /calculator?pm=username
    $pm = get_query_var('pm');

    $user = get_user_by('login', $pm);
    if ($user){
       $pm_user_id = $user->ID;
    }

    $fname = $_POST['wpcf-first-name'];
    $lname = $_POST['wpcf-last-name'];
    $address = $_POST['wpcf-address'];
    $zip_code = $_POST['wpcf-zip-code'];
    
    if (isset($_POST['wpcf-current-household-income'])) {
      if ($_POST['wpcf-current-household-income'] == 'per-week') {
        $income = $_POST['wpcf-household-income-per-week'] * 4.3;
      } else if ($_POST['wpcf-current-household-income'] == 'per-year') {
        $income = $_POST['wpcf-household-income-per-year'] / 12;
      } else if ($_POST['wpcf-current-household-income'] == 'none') {
        $income = 0;
      }
    }
    if (isset($_POST['wpcf-household-state-unemployment-income'])) {
      if ($_POST['wpcf-household-state-unemployment-income'] == 'per-week') {
        $state_unemployment = $_POST['wpcf-household-state-unemployment-income-per-week'] * 4.3;
      } else if ($_POST['wpcf-household-state-unemployment-income'] == 'none') {
        $state_unemployment = 0;
      }
    }
    if (isset($_POST['wpcf-household-federal-unemployment-income'])) {
      if ($_POST['wpcf-household-federal-unemployment-income'] == 'per-week') {
        $federal_unemployment = $_POST['wpcf-household-federal-unemployment-income-per-week'] * 4.3;
      } else if ($_POST['wpcf-household-federal-unemployment-income'] == 'none') {
        $federal_unemployment = 0;
      }
    }

    $total_income = $income + $state_unemployment + $federal_unemployment;
    $liquid_assets_user = $_POST['wpcf-liquid-assets'];
    
    $test_zip_code = '98513';
    $test_lname = 'Clark';
    $test_address = '7529 BRIANNA COURT SE';	

    // this will return an array like this: ['address_key'=>'_7529_BRIANNA__98513','estimated_value'=>'335304','equity_pct'=>'29','liquid_assets'=>'75000 to 99999'] OR like this ['error'=>'some error']
    $api_data = get_api_data($test_address, $test_zip_code, $test_lname);

    if (isset($api_data['error'])) {
      error_log( var_dump($api_data['error']) );
      // TODO: send to error page with custom message?
      die;
    } else {
      
      $address_key = $api_data['address_key'];
      $estimated_value = $api_data['estimated_value'];
      $equity_pct = $api_data['equity_pct'] / 100;
      $equity_value = $estimated_value * $equity_pct;
      $liquid_assets = $api_data['liquid_assets'];

      $summary = '%results%

USER DATA:
Total monthly income: $' . $total_income . '
Total monthly expenses: %expenses%
Liquid Assets: $' . $liquid_assets_user . '

THIRD PARTY DATA:
Home Equity Value: $' .$equity_value . '
Liquid Assets: $' . $liquid_assets;

      $iso_date = get_the_date('c');
      $my_post = array(
        'ID'            => $post_id,
        'post_author'   => $pm_user_id,
        'post_title'    => $fname ." ".$lname,
        'post_name'     => $lname ."_".$address_key."_".$iso_date,
      );

      // Update the post into the database
      wp_update_post( $my_post );
      update_post_meta($post_id, 'wpcf-address-key', $address_key);
      update_post_meta($post_id, 'wpcf-manager-summary', $summary, true);

    }
    
  }

  if ($form_data['id']==133) {
    
    if (isset($_POST['form_submit_2'])) {
    //if ( isset($_POST['form_check']) && $_POST['form_check']=="submitted" ) {
      $cid = get_query_var('cid');
      $summary = get_post_meta( $cid, 'wpcf-manager-summary', true );
      preg_match('/Total monthly income: [\$](.*)/', $summary, $matches);
      $total_income = $matches[1];

      $child_posts = toolset_get_related_posts( $cid, 'payment-obligations', array( 'query_by_role' => 'parent', 'return' => 'post_object' ) );
      $obligation_amount = 0;
      foreach ($child_posts as $child_post) {
        $obligation_amount += get_post_meta( $child_post->ID, 'wpcf-obligation-amount', true );
      }
      
      // "A non well formed numeric value encountered"
      $financial_result = $total_income - $obligation_amount;
      if ($financial_result > 0) {
        $result = 'Does Not Qualify';
      } else {
        $result = 'Qualifies';
      }
      
      $summary = str_replace("%results%", $result, $summary);
      $summary = str_replace("%expenses%", '$'.$obligation_amount, $summary);

      update_post_meta($cid, 'wpcf-manager-summary', $summary);
    
      wp_redirect( get_permalink( 169 ) );
      exit();
    }
    
    // [wpv-conditional if="('[wpv-search-term param="_success"]' NE '')"]
    
  }
  
  if ( $form_data['id']==115) {
      if ( !empty( $_POST['user_email'] ) && !empty( $_POST['user_pass'] ) ) {;
        $user = get_user_by( 'email', $_POST['user_email'] );

        $signon = array(
            'user_login'    => $user->user_login,
            'user_password' => $_POST['user_pass'],
            'remember'      => true
        );

        $login = wp_signon( $signon, false );

        if ( is_wp_error($login) ) {
            error_log( $login->get_error_message() );
        }
      }
    }
}


// connect to Data Axle API
function get_api_data($raw_address, $zip, $last_name) {	
	$api_data = [];
	
	$street_labels = ['apt','ave','avenue','bend','bluff','blvd','boulevard','cir','circl','circle','cou','court','cove','crossing','ct','dr','drive','e','east','freeway','glen','grove','lan','lane','ln','lndg','loop','n','ne','north','nw','overlook','parkway','pass','path','pkwy','pl','plac','place','point','pt','rd','ridge','road','row','run','s','se','south','sq','st','street','sw','ter','terr','terrac','terrace','tr','trai','trail','trl','turn','unit','w','way','west'];
	$street_pre_end_labels = ['ne','nw','se','sw','north','east','south','west','n','e','s','w'];	
	
	$raw_address = str_ireplace(",","",$raw_address);
	$raw_address = str_ireplace("'","",$raw_address);
	$raw_address = str_ireplace('.','',$raw_address);
	$address_parts = explode(' ',$raw_address);
	$last_part = $address_parts[count($address_parts)-1];
	$unit = '';
	$pre_address = '';
	if(stripos($last_part, '#')!==false) {
		$unit = str_ireplace("#","",$last_part);
		array_pop($address_parts);
	} elseif (is_numeric($last_part) or (strlen($last_part)==1 AND !in_array(strtolower($last_part),$street_labels))) {
		$unit = $last_part;
		array_pop($address_parts);
		if(strtolower($address_parts[count($address_parts)-1])=='unit') {
			array_pop($address_parts);
		}
	} elseif (in_array(strtolower($last_part),$street_pre_end_labels)) {
		array_pop($address_parts);
	}
	if(count($address_parts)>1) {
		$this_address = '';
		foreach($address_parts as $a_index=>$address_part) {
			if($a_index==0) {
				if (is_numeric($address_part)) {
					$street_number = $address_part;
				} else {
					$street_number = 0;
				}
			} elseif ($a_index==1) {
				if(!in_array(strtolower($address_part),$street_pre_end_labels)) {
					$this_address .= $address_part;
				} else {
					$pre_address = $address_part;
				}
			} elseif ($a_index==(count($address_parts)-1)) { // last part
				if(!in_array(strtolower($address_part),$street_labels)) {
					$this_address .= " ".$address_part;
				}
			} else {
				$this_address .= " ".$address_part;
			}
		}
	}

	if($street_number==0 OR $this_address=='') {
		$api_data['error'] = "Bad Address";	
		return $api_data;
	}	
	
	$api_data['address_key'] = $pre_address."_".$street_number."_".$this_address."_".$unit."_".$zip;	
	
	// authenticate data tree
	$username = urlencode("DTAPI_EquityExperts_Prod");
	$password = urlencode("EquityExpertsDTAPI1@Prod");
	$auth_fields = [
		'Username' => $username,
		'Password' => $password
	];
	$auth_postvars = '';
	foreach($auth_fields as $key=>$value) {
		$auth_postvars .= $key . "=" . $value . "&";
	}
	$curl = curl_init();
	curl_setopt_array($curl, array(
	  CURLOPT_PORT => "443",
	  CURLOPT_URL => "https://dtapi.datatree.com:443/api/Login/Authenticate",
	  CURLOPT_RETURNTRANSFER => true,
	  CURLOPT_CUSTOMREQUEST => "POST",
	  CURLOPT_POSTFIELDS => $auth_postvars,
	  CURLOPT_HTTPHEADER => array(
		"Content-Type: application/x-www-form-urlencoded",
		"cache-control: no-cache"
	  ),
	));
	$token = curl_exec($curl);
	$token = str_ireplace('"','',$token);
	curl_close($curl);

	//hit datatree
	$base_filters_array = [["FilterName"=>"StreetNumber","FilterOperator"=>"is","FilterValues"=>[$street_number],"FilterGroup"=>0],["FilterName"=>"StreetNames","FilterOperator"=>"is","FilterValues"=>[$this_address],"FilterGroup"=>0],["FilterName"=>"ZipCodeRange","FilterOperator"=>"is","FilterValues"=>[$zip],"FilterGroup"=>0]];

	$filters_array = $base_filters_array;
	if ($pre_address<>'') {
		array_push($filters_array,["FilterName"=>"StreetDir","FilterOperator"=>"is","FilterValues"=>[$pre_address],"FilterGroup"=>0]);
		if($unit<>'') {
			array_push($filters_array,["FilterName"=>"MailingUnit","FilterOperator"=>"is","FilterValues"=>[$unit],"FilterGroup"=>0]);		
		}				
		$fields = [
		'ReferenceId' => 'Export Template',
		'ProductNames' => 'PropertyDetailExport',
		'SpatialType' => 'Geometry',
		'MaxReturn' => 100,
		'Filters' => $filters_array
		];
		$postvars = json_encode($fields); 
		$curl = curl_init();
		curl_setopt_array($curl, array(
		  CURLOPT_PORT => "443",
		  CURLOPT_URL => "https://dtapi.datatree.com/api/Report/ExportReport",
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_CUSTOMREQUEST => "POST",
		  CURLOPT_POSTFIELDS => $postvars,
		  CURLOPT_HTTPHEADER => array(
			"Content-Type: application/json",
			"Accept: application/json",
			"cache-control: no-cache",
			"Authorization: ".$token
		  ),
		));
		$results = curl_exec($curl);
		curl_close($curl);
		$data = json_decode($results);			
	}

	if($data->MaxResultsCount>0) {
		$found_record = 1;
	} else {
		$filters_array = $base_filters_array;
		if($unit<>'') {
			array_push($filters_array,["FilterName"=>"MailingUnit","FilterOperator"=>"is","FilterValues"=>[$unit],"FilterGroup"=>0]);		
		}				
		$fields = [
			'ReferenceId' => 'Export Template',
			'ProductNames' => 'PropertyDetailExport',
			'SpatialType' => 'Geometry',
			'MaxReturn' => 100,
			'Filters' => $filters_array
		];
		$postvars = json_encode($fields); 
		$curl = curl_init();
		curl_setopt_array($curl, array(
		  CURLOPT_PORT => "443",
		  CURLOPT_URL => "https://dtapi.datatree.com/api/Report/ExportReport",
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_CUSTOMREQUEST => "POST",
		  CURLOPT_POSTFIELDS => $postvars,
		  CURLOPT_HTTPHEADER => array(
			"Content-Type: application/json",
			"Accept: application/json",
			"cache-control: no-cache",
			"Authorization: ".$token
		  ),
		));
		$results = curl_exec($curl);
		curl_close($curl);
		$data = json_decode($results);	
		if($data->MaxResultsCount>0) {
			$found_record = 1;	
		}					
	} 

	if($found_record) {
		$records = $data->Reports[0]->Data->PropertyData;
		foreach($records as $record) {
			$api_data['equity_pct'] = $record->HomeEquityPercentage;
			$api_data['estimated_value'] = $record->EstimatedValue;
			break;
		}		

		// grab data axle
		$curl = curl_init();
		curl_setopt_array($curl, array(
		  CURLOPT_URL => "https://data-axle.com/api/1/people/search",
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_ENCODING => "",
		  CURLOPT_MAXREDIRS => 10,
		  CURLOPT_TIMEOUT => 0,
		  CURLOPT_FOLLOWLOCATION => true,
		  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		  CURLOPT_CUSTOMREQUEST => "GET",
		  CURLOPT_POSTFIELDS =>"{ \"filter\": { \"connective\": \"and\", \"propositions\": [ { \"relation\": \"matches\", \"attribute\": \"street\", \"value\": \"".$this_address."\" }, { \"relation\": \"matches\", \"attribute\": \"zip\", \"value\": \"".$zip."\" }, { \"relation\": \"equals\", \"attribute\": \"last_name\", \"value\": \"".$last_name."\" } ] }}",
		  CURLOPT_HTTPHEADER => array(
			"X-AUTH-TOKEN: 2194a809598987f8b0e75d48",
			"Content-Type: text/plain"
		  ),
		));

		$response = curl_exec($curl);
		$data = json_decode($response);
		curl_close($curl);

		$family_fields = ['Family: Estimated Liquid Assets'=>'estimated_liquid_assets'];
		$record = $data->documents[0];

		foreach($family_fields as $label=>$path) {
			$the_value = $record->family->$path;
			$api_data['liquid_assets'] = generate_values($label,$the_value);
		}						
			
	} else {
		$api_data['error'] = "No matching result for that address";	
	}
	
	return $api_data;		
}


// manipulate range fields from Data Axle API
function generate_values($label,$the_value) {
	
	$fields_with_ranges = ['Real estate: Estimated Home Value','Mortgage: Estimated Home Equity','Mortgage: Estimated Loan To Value','Family: Estimated Income Range','Family: Estimated Liquid Assets','Family: Estimated Vacation Expenses','Family: Estimated Wealth'];
	
	if(is_array($the_value)) {
		if(in_array($label,$fields_with_ranges)) {
			$final_value = $the_value[0]." to ".$the_value[1];
		} else {
			$final_value = '';
			foreach($the_value as $ind=>$val) {
				if($final_value=='') {
					$final_value = $val;
				} else {
					$final_value .= ", ".$val;
				}
			}
		}
	} else {
		if($the_value=='') {
			$the_value = 'unknown';
		}
		$final_value = $the_value;		
	}
	return($final_value);
}